unit TWall;
{ ============================================================================
  Author:   Ben Menking (731422)
  Email:    bmenking@acm.org
  Type:     Delphi 2.0 Non-Visual Component
  Version:  1.0b
  Code:     Delphi Object Pascal
  Descript: Allows user to change Windows 95 wallpaper by passing in a
            filename and mode (tiled or centered).
  !NOTE!:   This component (compiled or source) is freeware that can be used
            by anyone.  If you use it for a commercial products, please give
            credit to me.  You may modify the source code to your desire and
            if you make any cool modifications, please send me the mods!! :)
  |Copyright Ben Menking 1997|
  ===========================================================================  }
interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, Registry;

type
  TWallPaperChanger = class(TComponent)
  private
    fActive: Boolean;
    fTiled: Boolean;
    fTimer: TTimer;
    fInterval: Longint;
    fFilename: String;
    fOnBeginChange: TNotifyEvent;
    fOnFinishChange: TNotifyEvent;
    function GetFileName: String;
    procedure SetActive( Value: Boolean );
    procedure SetFilename( const FName: String );
    procedure SetTiled( Value: Boolean );
    procedure SetInterval( Value: Longint );
  protected
    procedure OnChangeTimer( Sender: TObject );
    Procedure ChangeWallPaper( FName: String; IsTiled: Boolean );
  public
    constructor Create( AOwner: TComponent ); override;
    destructor Destroy; override;
    procedure ChangeNow;
  published
    property Active: Boolean read fActive write SetActive;
    property Tiled: Boolean read fTiled write SetTiled;
    property Interval: Longint read fInterval write SetInterval;
    property Filename: String read GetFileName write SetFileName;
    property OnBeginChange: TNotifyEvent read fOnBeginChange write fOnBeginChange;
    property OnFinishChange: TNotifyEvent read fOnFinishChange write fOnFinishChange;
  end;

procedure Register;

implementation


function TWallPaperChanger.GetFileName: String;
begin
  result := fFileName;
end;

procedure TWallPaperChanger.SetActive( Value: Boolean );
begin
   if Value <> fActive then
   begin
     fActive := Value;
     if not (csdesigning in ComponentState) then
     begin
       case Value of
         TRUE: fTimer.OnTimer := OnChangeTimer;
         FALSE: fTimer.OnTimer := nil;
       end;
       fTimer.Enabled := Value;
     end;
   end;
end;

procedure TWallPaperChanger.SetFilename( const FName: String );
begin
  if not (csdesigning in ComponentState) then
  begin
    if FileExists( FName ) then
    begin
      fFileName := FName;
    end;
  end
  else
    fFileName := FName;
end;

procedure TWallPaperChanger.SetTiled( Value: Boolean );
begin
  fTiled := Value;
end;

constructor TWallPaperChanger.create(aOwner : Tcomponent);
begin
  inherited create(aOwner);
  fTimer := TTimer.Create( self );
  fTimer.Enabled := false;
  fFileName := '';
  fInterval := fTimer.Interval;
  fTimer.OnTimer := nil;
end;

procedure TWallPaperChanger.SetInterval( Value: Longint );
begin
  fInterval := Value;
  fTimer.Interval := Value;
end;

procedure TWallPaperChanger.ChangeNow;
begin
  if Assigned(OnBeginChange) then OnBeginChange( self );
  ChangeWallpaper( GetFileName, fTiled );
  if Assigned(OnFinishChange) then OnFinishChange( self );
end;

procedure TWallPaperChanger.OnChangeTimer( Sender: TObject );
begin
  ChangeNow;
end;

destructor TWallPaperChanger.Destroy;
begin
  inherited destroy;
end;

procedure Register;
begin
  RegisterComponents('System', [TWallPaperChanger]);
end;

Procedure TWallPaperChanger.ChangeWallPaper( FName: String; IsTiled: Boolean );
type FRName = array[0..255] of Char;
var
  PasFileName: String;
  Ptr: ^FRName;
  Reg: TRegistry;
begin
  New( Ptr );
  Reg := TRegistry.Create;
  Reg.OpenKey( '\Control Panel\desktop', FALSE );
  if IsTiled then
  begin
    Reg.WriteString( 'TileWallPaper', '1' );
  end
  else
  begin
    Reg.WriteString( 'TileWallPaper', '0' );
  end;
  PasFileName := FName;
  StrPCopy( Ptr^, PasFileName );
  SystemParametersInfo( SPI_SetDeskWallPaper, 1, Ptr, SPIF_UPDATEINIFILE );
  Reg.OpenKey( '\Control Panel\desktop', FALSE );
  Reg.WriteString( 'Wallpaper', FName );
  DisPose( Ptr );
  Reg.Free;
end;
end.
